<?php

namespace Drupal\exchange_rates\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Config form to set base and conversion currencies.
 */
class ExchangeRatesSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'exchange_rates.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'exchange_rates_settings_form';
  }

  /**
   * List of all available currencies from exchangeratesapi.io.
   */
  public function currencyList() {
    return [
      'AUD' => $this->t('Australian Dollar (A$)'),
      'BGN' => $this->t('Bulgarian Lev (BGN)'),
      'BRL' => $this->t('Brazilian Real (R$)'),
      'CAD' => $this->t('Canadian Dollar (CA$)'),
      'CHF' => $this->t('Swiss Franc (CHF)'),
      'CNY' => $this->t('Chinese Yuan (CN¥)'),
      'CZK' => $this->t('Czech Republic Koruna (CZK)'),
      'DKK' => $this->t('Danish Krone (DKK)'),
      'EUR' => $this->t('Euro (€)'),
      'GBP' => $this->t('British Pound (£)'),
      'HKD' => $this->t('Hong Kong Dollar (HK$)'),
      'HRK' => $this->t('Croatian Kuna (HRK)'),
      'HUF' => $this->t('Hungarian Forint (HUF)'),
      'IDR' => $this->t('Indonesian Rupiah (IDR)'),
      'ILS' => $this->t('Israeli New Sheqel (₪)'),
      'INR' => $this->t('Indian Rupee (Rs.)'),
      'ISK' => $this->t('Icelandic Króna (ISK)'),
      'JPY' => $this->t('Japanese Yen (¥)'),
      'KRW' => $this->t('South Korean Won (₩)'),
      'MXN' => $this->t('Mexican Peso (MX$)'),
      'MYR' => $this->t('Malaysian Ringgit (MYR)'),
      'NOK' => $this->t('Norwegian Krone (NOK)'),
      'NZD' => $this->t('New Zealand Dollar (NZ$)'),
      'PHP' => $this->t('Philippine Peso (Php)'),
      'PLN' => $this->t('Polish Zloty (PLN)'),
      'RON' => $this->t('Romanian Leu (RON)'),
      'RUB' => $this->t('Russian Ruble (RUB)'),
      'SEK' => $this->t('Swedish Krona (SEK)'),
      'SGD' => $this->t('Singapore Dollar (SGD)'),
      'THB' => $this->t('Thai Baht (THB)'),
      'TRY' => $this->t('Turkish Lira (TRY)'),
      'USD' => $this->t('US Dollar ($)'),
      'ZAR' => $this->t('South African Rand (ZAR)'),
    ];
  }

  /**
   * Builds Exchange Rates settings form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('exchange_rates.settings');
    $total_conversion_currency = $form_state->get('total_conversion_currency');

    if ($total_conversion_currency === NULL) {
      $total_conversion_currency = count($config->get('conversion_currency'));
      $form_state->set('total_conversion_currency', $total_conversion_currency);
    }

    $form['#tree'] = TRUE;
    $form['fx_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Exchange Rates Settings'),
      '#prefix' => '<div id="exchange-rates-fieldset-wrapper">',
      '#suffix' => '</div>',
    ];

    $form['fx_fieldset']['base_currency'] = [
      '#type' => 'select',
      '#title' => $this->t('Base Currency:'),
      '#options' => $this->currencyList(),
      '#default_value' => $config->get('base_currency'),
    ];

    $form['fx_fieldset']['conversion_currency'][0] = [
      '#type' => 'select',
      '#title' => $this->t('Conversion Currencies:'),
      '#options' => $this->currencyList(),
      '#default_value' => $config->get('conversion_currency')[0],
    ];

    if ($total_conversion_currency > 1) {
      for ($i = 1; $i < $total_conversion_currency; $i++) {
        $form['fx_fieldset']['conversion_currency'][$i] = [
          '#type' => 'select',
          '#options' => $this->currencyList(),
          '#default_value' => $config->get('conversion_currency')[$i],
        ];
      }
    }

    $form['fx_fieldset']['actions'] = [
      '#type' => 'actions',
    ];

    if ($total_conversion_currency < count($this->currencyList())) {
      $form['fx_fieldset']['actions']['add'] = [
        '#type' => 'submit',
        '#value' => $this->t('Add Currency'),
        '#submit' => ['::addConversionCurrency'],
        '#ajax' => [
          'callback' => '::conversionCurrencyCallback',
          'wrapper' => 'exchange-rates-fieldset-wrapper',
        ],
      ];
    }

    if ($total_conversion_currency > 1) {
      $form['fx_fieldset']['actions']['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove Currency'),
        '#submit' => ['::removeConversionCurrency'],
        '#ajax' => [
          'callback' => '::conversionCurrencyCallback',
          'wrapper' => 'exchange-rates-fieldset-wrapper',
        ],
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * Ajax callback.
   */
  public function conversionCurrencyCallback(array &$form, FormStateInterface $form_state) {
    return $form['fx_fieldset'];
  }

  /**
   * Add new conversion currency to list.
   */
  public function addConversionCurrency(array &$form, FormStateInterface $form_state) {
    $form_state->set('total_conversion_currency', $form_state->get('total_conversion_currency') + 1);
    $form_state->setRebuild();
  }

  /**
   * Remove last conversion currency from list.
   */
  public function removeConversionCurrency(array &$form, FormStateInterface $form_state) {
    $total_conversion_currency = $form_state->get('total_conversion_currency');

    if ($total_conversion_currency > 1) {
      $form_state->set('total_conversion_currency', $total_conversion_currency - 1);
    }

    $form_state->setRebuild();
  }

  /**
   * Saves base currency and conversion currencies selected on form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory()->getEditable('exchange_rates.settings');
    $fx_fieldset = $form_state->getValue('fx_fieldset');

    $config->set('base_currency', $fx_fieldset['base_currency']);
    $config->set('conversion_currency', $fx_fieldset['conversion_currency']);
    $config->save();

    // Confirmation on form submission.
    $this->messenger()->addStatus($this->t('The Exchange Rates settings have been saved.'));
  }

}
